<?php

namespace Drupal\custom_table_operations\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Database;



/**
 * PUse to delete table records
 */
class DbTableForm extends EntityForm{

    public function __construct(EntityTypeManagerInterface $entityTypeManager){
        $this->entityTypeManager = $entityTypeManager;
    }

    /**
     * {@inheritdoc}
     */

     public static function create(ContainerInterface $containeer){
        return new static(
            $containeer->get('entity_type.manager')
        );
     }

     /**
      * {@inheritdoc}
      */
      public function form(array $form, FormStateInterface $form_state){

        $form = parent::form($form, $form_state);

        $dbtable = $this->entity;

        $form['label'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Table name'),
            '#maxlength' => 255,
            '#default_value' => $dbtable->label(),
            '#description' => $this->t("Enter table name."),
            '#required' => TRUE,
            '#ajax' => [
              'callback' => '::myAjaxCallback',
              'disable-refocus' => FALSE,  
              'event' => 'change', 
              'wrapper'=>'ajaxfield',
            ],
          ];
          $form['id'] = [
            '#type' => 'machine_name',
            '#default_value' => $dbtable->id(),
            '#machine_name' => [
              'exists' => [$this, 'exist'],
            ],
           '#disabled' => !$dbtable->isNew(),           
            
          ];

          

             
          $form['table_field_details'] = [
            '#type' => 'details',
            '#title' => $this->t('Table field settings'),
            '#open' => TRUE,
            // We set the ID of this fieldset to questions-fieldset-wrapper so the
            // AJAX command can replace it.
            '#attributes' => [
              'id' => 'ajaxfield',
              'class' => ['questions-wrapper'],
            ],
          ];
          $dbtable_name = $form_state->getValue('label')?$form_state->getValue('label') : $dbtable->label();

          if(isset($dbtable_name)){

            $conn = Database::getConnection();
            $schema = \Drupal::database()->schema();
            if(!$schema->tableExists($dbtable_name)){

              $this->messenger()->addMessage($this->t('%label table does not exists! Please enter correct table name.', [
                  '%label' => $dbtable_name,
                 // '%field' => $dbtable->primary_key,
                ]), 'error');
              }else{

                $header = array();
                $fields = $conn->query("DESCRIBE ".$dbtable_name)->fetchAll();
                foreach($fields as $key=>$field){
                  $header[$field->Field] = $field->Field;
              }
            //print_r($record);die;
    
                $form['table_field_details']['primary_key'] = [
                  '#type' => 'select',
                  '#title' => $this->t('Primary key for '.$dbtable_name),
                  '#default_value' => $dbtable->primary_key,
                  '#description' => $this->t("Select column name set as primary key."),
                  '#required' => TRUE,
                  '#options' => $header,
                ];
                $form['table_field_details']['field_list'] = [
                  '#type' => 'select',
                  '#title' => $this->t('Fields available in '.$dbtable_name),
                  '#default_value' => $dbtable->field_list,
                  '#description' => $this->t("Select fields/columns to be displayed."),
                  '#required' => TRUE,
                  '#options' => $header,
                  '#multiple' => TRUE,
                ];

              }
           
          }

          

          // You will need additional form elements for your custom properties.
          return $form;
        }

        public function myAjaxCallback(array $form, FormStateInterface $form_state) {
          
                return $form['table_field_details'];

        }
      
        /**
         * {@inheritdoc}
         */
        public function save(array $form, FormStateInterface $form_state) {
          $dbtable = $this->entity;

          $schema = \Drupal::database()->schema();

          if(!$schema->tableExists($dbtable->label()) || !$schema->fieldExists($dbtable->label(), $dbtable->primary_key)){

            $this->messenger()->addMessage($this->t('Either %label table or %field field does not exists!', [
                '%label' => $dbtable->label(),
                '%field' => $dbtable->primary_key,
              ]), 'error');

              $form_state->setRedirect('entity.dbtable.collection');
              return;
          }


          $status = $dbtable->save();
      
          if ($status === SAVED_NEW) {
            $this->messenger()->addMessage($this->t('The %label table created.', [
              '%label' => $dbtable->label(),
            ]));
          }
          else {
            $this->messenger()->addMessage($this->t('The %label dbtable updated.', [
              '%label' => $dbtable->label(),
            ]));
          }
      
          $form_state->setRedirect('entity.dbtable.collection');
        }
      
        /**
         * Helper function to check whether an dbtable configuration entity exists.
         */
        public function exist($id) {
          $entity = $this->entityTypeManager->getStorage('dbtable')->getQuery()
            ->condition('id', $id)
            ->execute();
          return (bool) $entity;
        }
      
}