<?php

namespace Drupal\custom_table_operations\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Link;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Url;
use Drupal\Core\Messenger;


/**
 * PUse to delete table records
 */
class TableContentForm extends FormBase{


    public function getFormId(){
        return 'dbtable_form';
    }

    public function buildForm(array $form, FormStateInterface $form_state){

        $route = \Drupal::routeMatch();
        $table = $route->getParameter('dbtable');
        $pk = $route->getParameter('pk');
        $cid = $route->getParameter('cid');

        $entity = \Drupal::entityTypeManager()
        ->getStorage('dbtable')
        ->load($table);
        $fields = $entity->field_list;
        $table_key = [$pk => $pk];
        $select_fields = array_merge($table_key, $fields);


        $conn = Database::getConnection();

        $record = [];

        if(isset($cid)){
            $query = $conn->select($table, 'm')->condition($pk, $cid)->fields('m', $select_fields);
            $record = $query->execute()->fetchAssoc();
        }
        foreach($select_fields as $field){

            $form[$field] = [
                '#type' => 'textfield',
                '#title' => $field,
                '#required' => True,
                '#default_value' => (isset($record[$field]) ? $record[$field] : ''),
            ];

        }
       

        $form['action'] = ['#type' => 'action',];

        $back_link = $add = Url::fromRoute('entity.dbtable.content', ['dbtable' => $table, 'pk' => $pk], []);

        

        $form['action']['submit'] = [
            '#type' => 'submit', 
            '#value' => t('Save'),
        ];

        $form['action']['cancel'] = [
            '#type' => 'html_tag',
            '#value' =>Link::fromTextAndUrl('Cancel', $back_link )->toString(),
            '#tag' => 'button',
            '#attributes' => ['class' => ['button']]
        ];

        return $form;



    }


    public function submitForm(array &$form, FormStateInterface $form_state){

        $route = \Drupal::routeMatch();
        $table = $route->getParameter('dbtable');
        $pk = $route->getParameter('pk');
        $cid = $route->getParameter('cid');

        $fields = $form_state->cleanValues()->getValues('');


        $query = \Drupal::database();
        
        if(isset($cid)){
            
            $update = $query->update($table)->fields($fields)->condition($pk, $cid);
            if($update->execute()){
                $this->messenger()->addMessage("Record has been updated.");
            }else{
                $this->messenger()->addMessage("There is some error!.", 'error');

            }
        }else{
            $check = $query->select($table, 'm')->condition($pk, $fields[$pk])->countQuery()->execute()->fetchField();
            if($check == 0){
                $add = $query->insert($table)->fields($fields)->execute();
                $this->messenger()->addMessage("Record has been created successfully.");

                
            }else{
                $this->messenger()->addMessage("Record with id ".$fields[$pk]." already exists.","error");

            }
        }

        $form_state->setRedirect('entity.dbtable.content', ['dbtable' => $table, 'pk' => $pk]);

    }
}